@@ -3,6 +3,8 @@
 
 #ifndef UNW_REMOTE_ONLY
 
+#if defined __aarch64__
+# include "tdep-aarch64/jmpbuf.h"
 #if defined __arm__
 # include "tdep-arm/jmpbuf.h"
 #elif defined __hppa__
@@ -19,6 +21,8 @@
 # include "tdep-x86/jmpbuf.h"
 #elif defined __x86_64__
 # include "tdep-x86_64/jmpbuf.h"
+#elif defined __tilegx__
+# include "tdep-tilegx/jmpbuf.h"
 #else
 # error "Unsupported arch"
 #endif
