@@ -1,7 +1,11 @@
 /* Provide a real file - not a symlink - as it would cause multiarch conflicts
    when multiple different arch releases are installed simultaneously.  */
 
-#if defined __arm__
+#ifndef UNW_REMOTE_ONLY
+
+#if defined __aarch64__
+# include "tdep-aarch64/libunwind_i.h"
+#elif defined __arm__
 # include "tdep-arm/libunwind_i.h"
 #elif defined __hppa__
 # include "tdep-hppa/libunwind_i.h"
@@ -13,10 +17,21 @@
 # include "tdep-ppc32/libunwind_i.h"
 #elif defined __powerpc64__
 # include "tdep-ppc64/libunwind_i.h"
+#elif defined __sh__
+# include "tdep-sh/libunwind_i.h"
 #elif defined __i386__
 # include "tdep-x86/libunwind_i.h"
 #elif defined __x86_64__
 # include "tdep-x86_64/libunwind_i.h"
+#elif defined __tilegx__
+# include "tdep-tilegx/libunwind_i.h"
 #else
 # error "Unsupported arch"
 #endif
+
+
+#else /* UNW_REMOTE_ONLY */
+
+# include "tdep-x86_64/libunwind_i.h"
+
+#endif /* UNW_REMOTE_ONLY */
