@@ -76,6 +76,9 @@ unw_is_signal_frame (unw_cursor_t *cursor)
            || w0 == THUMB_RT_SIGRETURN)
     return 2;
 
+  return 0;
+#elif defined(__QNX__)
+  /* Not supported yet */
   return 0;
 #else
   printf ("%s: implement me\n", __FUNCTION__);
