@@ -1,6 +1,6 @@
 /* libunwind - a platform-independent unwind library
-   Copyright (C) 2001-2002 Hewlett-Packard Co
-	Contributed by David Mosberger-Tang <davidm@hpl.hp.com>
+   Copyright (C) 2001-2002, 2005 Hewlett-Packard Co
+        Contributed by David Mosberger-Tang <davidm@hpl.hp.com>
 
 This file is part of libunwind.
 
@@ -23,12 +23,12 @@ LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.  */
 
-#include "internal.h"
+#include "libunwind_i.h"
 
 HIDDEN unw_dyn_info_list_t _U_dyn_info_list;
 
 PROTECTED unw_word_t
 _U_dyn_info_list_addr (void)
 {
-  return (unw_word_t) &_U_dyn_info_list;
+  return (unw_word_t) (uintptr_t) &_U_dyn_info_list;
 }
