@@ -1,6 +1,6 @@
 /* libunwind - a platform-independent unwind library
-   Copyright (C) 2003 Hewlett-Packard Co
-	Contributed by David Mosberger-Tang <davidm@hpl.hp.com>
+   Copyright (C) 2003-2004 Hewlett-Packard Co
+        Contributed by David Mosberger-Tang <davidm@hpl.hp.com>
 
 This file is part of libunwind.
 
@@ -33,3 +33,6 @@ _Unwind_GetCFA (struct _Unwind_Context *context)
   unw_get_reg (&context->cursor, UNW_REG_SP, &val);
   return val;
 }
+
+unsigned long __libunwind_Unwind_GetCFA (struct _Unwind_Context *)
+     ALIAS (_Unwind_GetCFA);
