@@ -1,6 +1,6 @@
 /* libunwind - a platform-independent unwind library
-   Copyright (C) 2003 Hewlett-Packard Co
-	Contributed by David Mosberger-Tang <davidm@hpl.hp.com>
+   Copyright (C) 2003-2004 Hewlett-Packard Co
+        Contributed by David Mosberger-Tang <davidm@hpl.hp.com>
 
 This file is part of libunwind.
 
@@ -34,3 +34,6 @@ _Unwind_GetRegionStart (struct _Unwind_Context *context)
   unw_get_proc_info (&context->cursor, &pi);
   return pi.start_ip;
 }
+
+unsigned long __libunwind_Unwind_GetRegionStart (struct _Unwind_Context *)
+     ALIAS (_Unwind_GetRegionStart);
