@@ -1,6 +1,6 @@
 /* libunwind - a platform-independent unwind library
-   Copyright (C) 2003 Hewlett-Packard Co
-	Contributed by David Mosberger-Tang <davidm@hpl.hp.com>
+   Copyright (C) 2003-2004 Hewlett-Packard Co
+        Contributed by David Mosberger-Tang <davidm@hpl.hp.com>
 
 This file is part of libunwind.
 
@@ -30,3 +30,6 @@ _Unwind_GetTextRelBase (struct _Unwind_Context *context)
 {
   return 0;
 }
+
+unsigned long __libunwind_Unwind_GetTextRelBase (struct _Unwind_Context *)
+     ALIAS (_Unwind_GetTextRelBase);
