@@ -1,6 +1,6 @@
 /* libunwind - a platform-independent unwind library
    Copyright (c) 2002-2004 Hewlett-Packard Development Company, L.P.
-	Contributed by David Mosberger-Tang <davidm@hpl.hp.com>
+        Contributed by David Mosberger-Tang <davidm@hpl.hp.com>
 
    Modified for x86_64 by Max Asbock <masbock@us.ibm.com>
 
@@ -27,29 +27,14 @@ WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.  */
 
 #include <stdlib.h>
 
+#include "offsets.h"
 #include "unwind_i.h"
 
 #ifndef UNW_REMOTE_ONLY
 
-#include <sys/syscall.h>
-
-/* sigreturn() is a no-op on x86_64 glibc.  */
-
-static NORETURN inline long
-my_rt_sigreturn (void *new_sp)
-{
-  __asm__ __volatile__ ("mov %0, %%rsp;"
-			"mov %1, %%rax;"
-			"syscall"
-			:: "r"(new_sp), "i"(SYS_rt_sigreturn)
-			: "memory");
-  abort ();
-}
-
 HIDDEN inline int
 x86_64_local_resume (unw_addr_space_t as, unw_cursor_t *cursor, void *arg)
 {
-#if defined(__linux)
   struct cursor *c = (struct cursor *) cursor;
   ucontext_t *uc = c->uc;
 
@@ -61,21 +46,15 @@ x86_64_local_resume (unw_addr_space_t as, unw_cursor_t *cursor, void *arg)
 
   if (unlikely (c->sigcontext_format != X86_64_SCF_NONE))
     {
-      struct sigcontext *sc = (struct sigcontext *) c->sigcontext_addr;
-
-      Debug (8, "resuming at ip=%llx via sigreturn(%p)\n",
-	     (unsigned long long) c->dwarf.ip, sc);
-      my_rt_sigreturn (sc);
+      x86_64_sigreturn(cursor);
+      abort();
     }
   else
     {
       Debug (8, "resuming at ip=%llx via setcontext()\n",
-	     (unsigned long long) c->dwarf.ip);
-      _Ux86_64_setcontext (uc);
+             (unsigned long long) c->dwarf.ip);
+      setcontext (uc);
     }
-#else
-# warning Implement me!
-#endif
   return -UNW_EINVAL;
 }
 
@@ -88,9 +67,9 @@ static inline int
 establish_machine_state (struct cursor *c)
 {
   int (*access_reg) (unw_addr_space_t, unw_regnum_t, unw_word_t *,
-		     int write, void *);
+                     int write, void *);
   int (*access_fpreg) (unw_addr_space_t, unw_regnum_t, unw_fpreg_t *,
-		       int write, void *);
+                       int write, void *);
   unw_addr_space_t as = c->dwarf.as;
   void *arg = c->dwarf.as_arg;
   unw_fpreg_t fpval;
@@ -106,15 +85,15 @@ establish_machine_state (struct cursor *c)
     {
       Debug (16, "copying %s %d\n", unw_regname (reg), reg);
       if (unw_is_fpreg (reg))
-	{
-	  if (tdep_access_fpreg (c, reg, &fpval, 0) >= 0)
-	    (*access_fpreg) (as, reg, &fpval, 1, arg);
-	}
+        {
+          if (tdep_access_fpreg (c, reg, &fpval, 0) >= 0)
+            (*access_fpreg) (as, reg, &fpval, 1, arg);
+        }
       else
-	{
-	  if (tdep_access_reg (c, reg, &val, 0) >= 0)
-	    (*access_reg) (as, reg, &val, 1, arg);
-	}
+        {
+          if (tdep_access_reg (c, reg, &val, 0) >= 0)
+            (*access_reg) (as, reg, &val, 1, arg);
+        }
     }
   return 0;
 }
@@ -131,5 +110,5 @@ unw_resume (unw_cursor_t *cursor)
     return ret;
 
   return (*c->dwarf.as->acc.resume) (c->dwarf.as, (unw_cursor_t *) c,
-				     c->dwarf.as_arg);
+                                     c->dwarf.as_arg);
 }
