@@ -25,6 +25,7 @@ WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.  */
 #include "unwind_i.h"
 #include "ucontext_i.h"
 #include <signal.h>
+#include <limits.h>
 
 #pragma weak pthread_once
 #pragma weak pthread_key_create
@@ -34,30 +35,39 @@ WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.  */
 /* Initial hash table size. Table expands by 2 bits (times four). */
 #define HASH_MIN_BITS 14
 
-/* There's not enough space to store RIP's location in a signal
-   frame, but we can calculate it relative to RBP's (or RSP's)
-   position in mcontext structure.  Note we don't want to use
-   the UC_MCONTEXT_GREGS_* directly since we rely on DWARF info. */
-#define dRIP (UC_MCONTEXT_GREGS_RIP - UC_MCONTEXT_GREGS_RBP)
-
 typedef struct
 {
   unw_tdep_frame_t *frames;
   size_t log_size;
   size_t used;
+  size_t dtor_count;  /* Counts how many times our destructor has already
+                         been called. */
 } unw_trace_cache_t;
 
 static const unw_tdep_frame_t empty_frame = { 0, UNW_X86_64_FRAME_OTHER, -1, -1, 0, -1, -1 };
-static pthread_mutex_t trace_init_lock = PTHREAD_MUTEX_INITIALIZER;
+static define_lock (trace_init_lock);
 static pthread_once_t trace_cache_once = PTHREAD_ONCE_INIT;
+static sig_atomic_t trace_cache_once_happen;
 static pthread_key_t trace_cache_key;
 static struct mempool trace_cache_pool;
+static __thread  unw_trace_cache_t *tls_cache;
+static __thread  int tls_cache_destroyed;
 
 /* Free memory for a thread's trace cache. */
 static void
 trace_cache_free (void *arg)
 {
   unw_trace_cache_t *cache = arg;
+  if (++cache->dtor_count < PTHREAD_DESTRUCTOR_ITERATIONS)
+  {
+    /* Not yet our turn to get destroyed. Re-install ourselves into the key. */
+    pthread_setspecific(trace_cache_key, cache);
+    Debug(5, "delayed freeing cache %p (%zx to go)\n", cache,
+          PTHREAD_DESTRUCTOR_ITERATIONS - cache->dtor_count);
+    return;
+  }
+  tls_cache_destroyed = 1;
+  tls_cache = NULL;
   munmap (cache->frames, (1u << cache->log_size) * sizeof(unw_tdep_frame_t));
   mempool_free (&trace_cache_pool, cache);
   Debug(5, "freed cache %p\n", cache);
@@ -69,6 +79,7 @@ trace_cache_init_once (void)
 {
   pthread_key_create (&trace_cache_key, &trace_cache_free);
   mempool_init (&trace_cache_pool, sizeof (unw_trace_cache_t), 0);
+  trace_cache_once_happen = 1;
 }
 
 static unw_tdep_frame_t *
@@ -78,7 +89,7 @@ trace_cache_buckets (size_t n)
   size_t i;
 
   GET_MEMORY(frames, n * sizeof (unw_tdep_frame_t));
-  if (likely(frames != 0))
+  if (likely(frames != NULL))
     for (i = 0; i < n; ++i)
       frames[i] = empty_frame;
 
@@ -93,21 +104,32 @@ trace_cache_create (void)
 {
   unw_trace_cache_t *cache;
 
+  if (tls_cache_destroyed)
+  {
+    /* The current thread is in the process of exiting. Don't recreate
+       cache, as we wouldn't have another chance to free it. */
+    Debug(5, "refusing to reallocate cache: "
+             "thread-locals are being deallocated\n");
+    return NULL;
+  }
+
   if (! (cache = mempool_alloc(&trace_cache_pool)))
   {
     Debug(5, "failed to allocate cache\n");
-    return 0;
+    return NULL;
   }
 
   if (! (cache->frames = trace_cache_buckets(1u << HASH_MIN_BITS)))
   {
     Debug(5, "failed to allocate buckets\n");
     mempool_free(&trace_cache_pool, cache);
-    return 0;
+    return NULL;
   }
 
   cache->log_size = HASH_MIN_BITS;
   cache->used = 0;
+  cache->dtor_count = 0;
+  tls_cache_destroyed = 0;  /* Paranoia: should already be 0. */
   Debug(5, "allocated cache %p\n", cache);
   return cache;
 }
@@ -135,16 +157,36 @@ trace_cache_expand (unw_trace_cache_t *cache)
   return 0;
 }
 
-static __thread  unw_trace_cache_t *tls_cache;
+static unw_trace_cache_t *
+trace_cache_get_unthreaded (void)
+{
+  unw_trace_cache_t *cache;
+  intrmask_t saved_mask;
+  static unw_trace_cache_t *global_cache = NULL;
+  lock_acquire (&trace_init_lock, saved_mask);
+  if (! global_cache)
+  {
+    mempool_init (&trace_cache_pool, sizeof (unw_trace_cache_t), 0);
+    global_cache = trace_cache_create ();
+  }
+  cache = global_cache;
+  lock_release (&trace_init_lock, saved_mask);
+  Debug(5, "using cache %p\n", cache);
+  return cache;
+}
 
 /* Get the frame cache for the current thread. Create it if there is none. */
 static unw_trace_cache_t *
 trace_cache_get (void)
 {
   unw_trace_cache_t *cache;
-  if (likely (pthread_once != 0))
+  if (likely (pthread_once != NULL))
   {
     pthread_once(&trace_cache_once, &trace_cache_init_once);
+    if (!trace_cache_once_happen)
+    {
+      return trace_cache_get_unthreaded();
+    }
     if (! (cache = tls_cache))
     {
       cache = trace_cache_create();
@@ -156,18 +198,7 @@ trace_cache_get (void)
   }
   else
   {
-    intrmask_t saved_mask;
-    static unw_trace_cache_t *global_cache = 0;
-    lock_acquire (&trace_init_lock, saved_mask);
-    if (! global_cache)
-    {
-      mempool_init (&trace_cache_pool, sizeof (unw_trace_cache_t), 0);
-      global_cache = trace_cache_create ();
-    }
-    cache = global_cache;
-    lock_release (&trace_init_lock, saved_mask);
-    Debug(5, "using cache %p\n", cache);
-    return cache;
+    return trace_cache_get_unthreaded();
   }
 }
 
@@ -181,11 +212,11 @@ trace_cache_get (void)
    highly unusual unwind info which uses these creatively. */
 static unw_tdep_frame_t *
 trace_init_addr (unw_tdep_frame_t *f,
-		 unw_cursor_t *cursor,
-		 unw_word_t cfa,
-		 unw_word_t rip,
-		 unw_word_t rbp,
-		 unw_word_t rsp)
+                 unw_cursor_t *cursor,
+                 unw_word_t cfa,
+                 unw_word_t rip,
+                 unw_word_t rbp,
+                 unw_word_t rsp)
 {
   struct cursor *c = (struct cursor *) cursor;
   struct dwarf_cursor *d = &c->dwarf;
@@ -227,9 +258,9 @@ trace_init_addr (unw_tdep_frame_t *f,
     f->last_frame = -1;
 
   Debug (3, "frame va %lx type %d last %d cfa %s+%d rbp @ cfa%+d rsp @ cfa%+d\n",
-	 f->virtual_address, f->frame_type, f->last_frame,
-	 f->cfa_reg_rsp ? "rsp" : "rbp", f->cfa_reg_offset,
-	 f->rbp_cfa_offset, f->rsp_cfa_offset);
+         f->virtual_address, f->frame_type, f->last_frame,
+         f->cfa_reg_rsp ? "rsp" : "rbp", f->cfa_reg_offset,
+         f->rbp_cfa_offset, f->rsp_cfa_offset);
 
   return f;
 }
@@ -240,11 +271,11 @@ trace_init_addr (unw_tdep_frame_t *f,
    frame cache slot which describes RIP. */
 static unw_tdep_frame_t *
 trace_lookup (unw_cursor_t *cursor,
-	      unw_trace_cache_t *cache,
-	      unw_word_t cfa,
-	      unw_word_t rip,
-	      unw_word_t rbp,
-	      unw_word_t rsp)
+              unw_trace_cache_t *cache,
+              unw_word_t cfa,
+              unw_word_t rip,
+              unw_word_t rbp,
+              unw_word_t rsp)
 {
   /* First look up for previously cached information using cache as
      linear probing hash table with probe step of 1.  Majority of
@@ -285,7 +316,7 @@ trace_lookup (unw_cursor_t *cursor,
   if (unlikely(addr || cache->used >= cache_size / 2))
   {
     if (unlikely(trace_cache_expand (cache) < 0))
-      return 0;
+      return NULL;
 
     cache_size = 1u << cache->log_size;
     slot = ((rip * 0x9e3779b97f4a7c16) >> 43) & (cache_size-1);
@@ -406,7 +437,7 @@ tdep_trace (unw_cursor_t *cursor, void **buffer, int *size)
   {
     rip -= d->use_prev_instr;
     Debug (2, "depth %d cfa 0x%lx rip 0x%lx rsp 0x%lx rbp 0x%lx\n",
-	   depth, cfa, rip, rsp, rbp);
+           depth, cfa, rip, rsp, rbp);
 
     /* See if we have this address cached.  If not, evaluate enough of
        the dwarf unwind information to fill the cache line data, or to
@@ -449,7 +480,7 @@ tdep_trace (unw_cursor_t *cursor, void **buffer, int *size)
       cfa = (f->cfa_reg_rsp ? rsp : rbp) + f->cfa_reg_offset;
       ACCESS_MEM_FAST(ret, c->validate, d, cfa - 8, rip);
       if (likely(ret >= 0) && likely(f->rbp_cfa_offset != -1))
-	ACCESS_MEM_FAST(ret, c->validate, d, cfa + f->rbp_cfa_offset, rbp);
+        ACCESS_MEM_FAST(ret, c->validate, d, cfa + f->rbp_cfa_offset, rbp);
 
       /* Don't bother reading RSP from DWARF, CFA becomes new RSP. */
       rsp = cfa;
@@ -459,16 +490,13 @@ tdep_trace (unw_cursor_t *cursor, void **buffer, int *size)
       break;
 
     case UNW_X86_64_FRAME_SIGRETURN:
-      /* Advance standard signal frame, whose CFA points above saved
-         registers (ucontext) among other things.  We know the info
-	 is stored at some unknown constant offset off inner frame's
-	 CFA.  We determine the actual offset from DWARF unwind info. */
-      cfa = cfa + f->cfa_reg_offset;
-      ACCESS_MEM_FAST(ret, c->validate, d, cfa + f->rbp_cfa_offset + dRIP, rip);
+      cfa = cfa + f->cfa_reg_offset; /* cfa now points to ucontext_t.  */
+
+      ACCESS_MEM_FAST(ret, c->validate, d, cfa + UC_MCONTEXT_GREGS_RIP, rip);
       if (likely(ret >= 0))
-        ACCESS_MEM_FAST(ret, c->validate, d, cfa + f->rbp_cfa_offset, rbp);
+        ACCESS_MEM_FAST(ret, c->validate, d, cfa + UC_MCONTEXT_GREGS_RBP, rbp);
       if (likely(ret >= 0))
-        ACCESS_MEM_FAST(ret, c->validate, d, cfa + f->rsp_cfa_offset, rsp);
+        ACCESS_MEM_FAST(ret, c->validate, d, cfa + UC_MCONTEXT_GREGS_RSP, rsp);
 
       /* Resume stack at signal restoration point. The stack is not
          necessarily continuous here, especially with sigaltstack(). */
@@ -480,14 +508,14 @@ tdep_trace (unw_cursor_t *cursor, void **buffer, int *size)
 
     default:
       /* We cannot trace through this frame, give up and tell the
-	 caller we had to stop.  Data collected so far may still be
-	 useful to the caller, so let it know how far we got.  */
+         caller we had to stop.  Data collected so far may still be
+         useful to the caller, so let it know how far we got.  */
       ret = -UNW_ESTOPUNWIND;
       break;
     }
 
     Debug (4, "new cfa 0x%lx rip 0x%lx rsp 0x%lx rbp 0x%lx\n",
-	   cfa, rip, rsp, rbp);
+           cfa, rip, rsp, rbp);
 
     /* If we failed or ended up somewhere bogus, stop. */
     if (unlikely(ret < 0 || rip < 0x4000))
