@@ -28,7 +28,7 @@ WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.  */
 #ifdef HAVE_TTRACE
 
 int
-main (int argc, char **argv)
+main (void)
 {
   printf ("FAILURE: ttrace() not supported yet\n");
   return -1;
@@ -73,7 +73,7 @@ static struct UPT_info *ui;
 static int killed;
 
 void
-do_backtrace (pid_t target_pid)
+do_backtrace (void)
 {
   unw_word_t ip, sp, start_ip = 0, off;
   int n = 0, ret;
@@ -178,7 +178,7 @@ main (int argc, char **argv)
 
   if (argc == 1)
     {
-      char *args[] = { "self", "/bin/ls", "/usr", NULL };
+      static char *args[] = { "self", "/bin/ls", "/usr", NULL };
 
       /* automated test case */
       argv = args;
@@ -204,6 +204,8 @@ main (int argc, char **argv)
 	  ++optind, print_names = 0;
 	else
 	  fprintf(stderr, "unrecognized option: %s\n", argv[optind++]);
+        if (optind >= argc)
+          break;
       }
 
   target_pid = fork ();
@@ -221,6 +223,11 @@ main (int argc, char **argv)
 #else
 #error Trace me
 #endif
+
+      if ((argc > 1) && (optind == argc)) {
+        fprintf(stderr, "Need to specify a command line for the child\n");
+        exit (-1);
+      }
       execve (argv[optind], argv + optind, environ);
       _exit (-1);
     }
@@ -230,7 +237,7 @@ main (int argc, char **argv)
 
   while (nerrors <= nerrors_max)
     {
-      pid = wait4 (-1, &status,  0, 0);
+      pid = wait4 (-1, &status, 0, NULL);
       if (pid == -1)
 	{
 	  if (errno == EINTR)
@@ -294,11 +301,19 @@ main (int argc, char **argv)
 #endif
 	  else
 	    {
-	      do_backtrace (target_pid);
+	      do_backtrace ();
 #if HAVE_DECL_PTRACE_SINGLESTEP
-	      ptrace (PTRACE_SINGLESTEP, target_pid, 0, pending_sig);
+	      if (ptrace (PTRACE_SINGLESTEP, target_pid, 0, pending_sig) < 0)
+          {
+            panic ("ptrace(PTRACE_SINGLESTEP) failed (errno=%d)\n", errno);
+            killed = 1;
+          }
 #elif HAVE_DECL_PT_STEP
-	      ptrace (PT_STEP, target_pid, (caddr_t)1, pending_sig);
+	      if (ptrace (PT_STEP, target_pid, (caddr_t)1, pending_sig) < 0)
+          {
+            panic ("ptrace(PT_STEP) failed (errno=%d)\n", errno);
+            killed = 1;
+          }
 #else
 #error Singlestep me
 #endif
@@ -307,7 +322,7 @@ main (int argc, char **argv)
 
 	case SYSCALL:
 	  if (!state)
-	    do_backtrace (target_pid);
+	    do_backtrace ();
 	  state ^= 1;
 #if HAVE_DECL_PTRACE_SYSCALL
 	  ptrace (PTRACE_SYSCALL, target_pid, 0, pending_sig);
@@ -319,7 +334,7 @@ main (int argc, char **argv)
 	  break;
 
 	case INSTRUCTION:
-	  do_backtrace (target_pid);
+	  do_backtrace ();
 #if HAVE_DECL_PTRACE_SINGLESTEP
 	      ptrace (PTRACE_SINGLESTEP, target_pid, 0, pending_sig);
 #elif HAVE_DECL_PT_STEP
@@ -334,6 +349,7 @@ main (int argc, char **argv)
     }
 
   _UPT_destroy (ui);
+  unw_destroy_addr_space (as);
 
   if (nerrors)
     {
